"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const ErrorString_1 = require("../../shared/ErrorString");
const InvalidOperationException_1 = require("../../shared/InvalidOperationException");
/**
 * Returns the last element of a sequence.
 * If predicate is specified, the last element of a sequence that satisfies a specified condition.
 * @param source An IParallelEnumerable<T> to return the last element of.
 * @param predicate A function to test each element for a condition. Optional.
 * @throws {InvalidOperationException} The source sequence is empty.
 * @returns The value at the last position in the source sequence
 * or the last element in the sequence that passes the test in the specified predicate function.
 */
function last(source, predicate) {
    if (predicate) {
        return last_2(source, predicate);
    }
    else {
        return last_1(source);
    }
}
exports.last = last;
function last_1(source) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const dataFunc = source.dataFunc;
        switch (dataFunc.type) {
            case 0 /* PromiseToArray */: {
                const values = yield dataFunc.generator();
                if (values.length === 0) {
                    throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoElements);
                }
                else {
                    return values[values.length - 1];
                }
            }
            case 1 /* ArrayOfPromises */: {
                const promises = dataFunc.generator();
                if (promises.length === 0) {
                    throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoElements);
                }
                else {
                    return yield promises[promises.length - 1];
                }
            }
            case 2 /* PromiseOfPromises */: {
                const promises = yield dataFunc.generator();
                if (promises.length === 0) {
                    throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoElements);
                }
                else {
                    return yield promises[promises.length - 1];
                }
            }
        }
    });
}
function last_2(source, predicate) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        const dataFunc = source.dataFunc;
        switch (dataFunc.type) {
            case 0 /* PromiseToArray */: {
                const values = yield dataFunc.generator();
                // Promise Array - Predicate
                for (let i = values.length - 1; i >= 0; i--) {
                    const value = values[i];
                    if (predicate(value)) {
                        return value;
                    }
                }
                break;
            }
            case 1 /* ArrayOfPromises */: {
                const promises = dataFunc.generator();
                // Promise Array - Predicate
                for (let i = promises.length - 1; i >= 0; i--) {
                    const value = yield promises[i];
                    if (predicate(value)) {
                        return value;
                    }
                }
                break;
            }
            case 2 /* PromiseOfPromises */: {
                const promises = yield dataFunc.generator();
                // Promise Array - Predicate
                for (let i = promises.length - 1; i >= 0; i--) {
                    const value = yield promises[i];
                    if (predicate(value)) {
                        return value;
                    }
                }
                break;
            }
        }
        throw new InvalidOperationException_1.InvalidOperationException(ErrorString_1.ErrorString.NoMatch);
    });
}
